@@ -1,22 +1,11 @@
 # __main__.py
 
 import sys
+import importlib
 import pandas as pd
 from pathlib import Path
 from fba import __version__
-from fba.levenshtein import extract_feature_barcoding_fastss
-from fba.map import map_feature_barcoding
-from fba.filter import filter_matching
-from fba.count import generate_matrix
-from fba.qc import (
-    summarize_sequence_content,
-    summarize_barcode_positions,
-    analyze_bulk
-)
-from fba.regex import extract_feature_barcoding_regex
 from fba.parsers import parse_args
-from fba.demultiplex import demultiplex_feature_barcoding
-from fba.kallisto import run_kallisto
 from fba.utils import open_by_suffix, get_logger
 
 
@@ -52,6 +41,7 @@ def main():
 
     if (args.command == 'extract'):
         logger.info('Using extract subcommand ...')
+        m = importlib.import_module(name='fba.levenshtein')
 
         with open_by_suffix(file_name=args.output, mode='w') as f:
 
@@ -66,7 +56,7 @@ def main():
                 ]
             ) + '\n')
 
-            for out in extract_feature_barcoding_fastss(
+            for out in m.extract_feature_barcoding_fastss(
                     read1_file=args.read1,
                     read2_file=args.read2,
                     cb_file=args.whitelist,
@@ -85,8 +75,9 @@ def main():
 
     elif (args.command == 'map'):
         logger.info('Using map subcommand ...')
+        m = importlib.import_module(name=f'fba.{args.command}')
 
-        matrix_featurecount = map_feature_barcoding(
+        matrix_featurecount = m.map_feature_barcoding(
             read1_file=args.read1,
             read2_file=args.read2,
             cb_file=args.whitelist,
@@ -111,8 +102,9 @@ def main():
 
     elif (args.command == 'filter'):
         logger.info('Using filter subcommand ...')
+        m = importlib.import_module(name=f'fba.{args.command}')
 
-        _ = filter_matching(
+        _ = m.filter_matching(
             matching_file=args.input,
             filtered_matching_file=args.output,
             cb_pos_start=args.cell_barcode_pos_start,
@@ -132,8 +124,9 @@ def main():
 
     elif (args.command == 'count'):
         logger.info('Using count subcommand ...')
+        m = importlib.import_module(name=f'fba.{args.command}')
 
-        matrix_featurecount = generate_matrix(
+        matrix_featurecount = m.generate_matrix(
             matching_file=args.input,
             umi_pos_start=args.umi_pos_start,
             umi_length=args.umi_length,
@@ -150,7 +143,8 @@ def main():
     elif (args.command == 'demultiplex'):
         logger.info('Using demultiplex subcommand ...')
 
-        _ = demultiplex_feature_barcoding(
+        m = importlib.import_module(name=f'fba.{args.command}')
+        _ = m.demultiplex_feature_barcoding(
             matrix_featurecount_file=args.input,
             output_directory=args.output_directory,
             q=args.quantile,
@@ -163,6 +157,7 @@ def main():
 
     elif (args.command == 'qc'):
         logger.info('Using qc subcommand ...')
+        m = importlib.import_module(name=f'fba.{args.command}')
 
         if not isinstance(args.num_reads, int):
             if args.num_reads.isdigit():
@@ -175,7 +170,7 @@ def main():
             num_reads = args.num_reads
 
         if args.read1:
-            _ = summarize_sequence_content(
+            _ = m.summarize_sequence_content(
                 read1_file=args.read1,
                 read2_file=args.read2,
                 num_reads=num_reads,
@@ -199,7 +194,8 @@ def main():
                     ]
                 ) + '\n')
 
-                for out in extract_feature_barcoding_regex(
+                n = importlib.import_module(name='fba.regex')
+                for out in n.extract_feature_barcoding_regex(
                         read1_file=args.read1,
                         read2_file=args.read2,
                         cb_file=args.whitelist,
@@ -216,7 +212,7 @@ def main():
 
                     f.write(out + '\n')
 
-            _ = summarize_barcode_positions(
+            _ = m.summarize_barcode_positions(
                 matching_file=OUTPUT_FILE,
                 output_directory=args.output_directory)
 
@@ -231,7 +227,7 @@ def main():
                 'Skipping arguments: "-1", "-w", "-cb_m", "-r1_coords"'
             )
 
-            fb_frequency = analyze_bulk(
+            fb_frequency = m.analyze_bulk(
                 read_file=args.read2,
                 read_coords=args.read2_coords,
                 fb_file=args.feature_ref,
@@ -258,9 +254,10 @@ def main():
         logger.info('Done.')
 
     elif (args.command == 'kallisto_wrapper'):
-
         logger.info('Using kallisto_wrapper subcommand ...')
-        matrix_featurecount = run_kallisto(
+        m = importlib.import_module(name='fba.kallisto')
+
+        matrix_featurecount = m.run_kallisto(
             read1_file=args.read1,
             read2_file=args.read2,
             cb_file=args.whitelist,
